	.include "MSP430F44X.h"

LCD_DIV_32		.equ	000h
LCD_DIV_64		.equ	008h
LCD_DIV_128		.equ	010h
LCD_DIV_256		.equ	018h

XCAP18PF		.equ 	030h

RAM_orig  .set		 00200h		 ; Free Memory start address
ROM_orig  .set		 0E000h		 ; ROM start address
USER_END  .set		 0FFFFh		 ; RAM end address
SP_orig   .set		 00280h		 ; stackpointer

;************************************************************************
; Reset : Initialize processor
;************************************************************************

;	.sect "MAIN",ROM_orig
RESET
	mov #SP_orig,SP				   ; initialize stackpointer
	mov #(WDT_HOLD+WDT_WRKEY),&WDTCTL  ; Stop Watchdog Timer
	bis.b #XCAP18PF,&FLL_CTL0       ;set load capacitance for xtal

	; Initialize UART
	call #Init_UART

	; Setup Basic Timer for LCD operation
	mov.b #(LCD_DIV_64+023h),&BTCTL

	; Turn on Basic Timer interrupts
	bis.b #BTIE,&IE2

	; Setup module functions
	mov.b #032h,&P1SEL
	mov.b #000h,&P2SEL
	mov.b #000h,&P3SEL
	mov.b #0C3h,&P4SEL		; LCD/UART1
	mov.b #0FFh,&P5SEL		; LCD

	; Setup LCD
	mov.b #(LCDLASTSEG35|LCDMODE_4MUX|LCDSHOW|LCDMODULEON),&LCDCTL
	; Setup LCD data
	mov.b #000h,&LCDMEM1	; S1:S0	
	mov.b #080h,&LCDMEM2	; S3:S2

	; 78deg F
	mov.b #072h,&LCDMEM3	; S5:S4
	mov.b #033h,&LCDMEM4	; S7:S6
	mov.b #0F7h,&LCDMEM5	; S9:S8
	mov.b #007h,&LCDMEM6	; S11:S10

	mov.b #0E2h,&LCDMEM7	; S13:S12
	mov.b #044h,&LCDMEM8	; S15:S14

	mov.b #062h,&LCDMEM9	; S17:S16
	mov.b #044h,&LCDMEM10	; S19:S18

	mov.b #062h,&LCDMEM11	; S21:S20
	mov.b #044h,&LCDMEM12	; S23:S22

	mov.b #0E2h,&LCDMEM13	; S25:S24
	mov.b #064h,&LCDMEM14	; S27:S26
;	mov.b #001h,&LCDMEM13	; S25:S24	; Alternate 'A'
;	mov.b #06Ch,&LCDMEM14	; S27:S26

	mov.b #0C2h,&LCDMEM15	; S29:S28
	mov.b #064h,&LCDMEM16	; S31:S30

	mov.b #062h,&LCDMEM17	; S33:S32
	mov.b #0F4h,&LCDMEM18	; S35:S34

	; Setup LED outputs
	bis.b #0A0h,&P3DIR

	; Clear LEDs
	bic.b #0A0h,&P3OUT
	; Setup Blinky
	mov.b #033h,&P1DIR

	; Delay
	mov #08000h,R4
DelayLoopTop
	dec R4
	jnz DelayLoopTop

	; Setup P2.1, P2.2, and P2.3 as inputs
;	mov.b #031h,&P2DIR

	; Initialize accessory state
	mov #Display_0,R4

	eint

;******************************************************************************
; Mainloop
;******************************************************************************
LoopTop
	xor.b #020h,&P3OUT    ; Toggle LED
	bic.b #040h,&P1OUT	  ; Clear P1.6

	; Go to sleep
	bis #(CPUOFF+SCG1+SCG0),SR	
	bis.b #040h,&P1OUT	  ; Set P1.6

LoopDemoStuff_Next

	; Demo the symbology
	call R4

	; Check the switch input
	bit.b #040h,&P1IN
	jnz SwitchOff
	
SwitchOn
	; Turn on second LED
	bic.b #080h,&P3OUT
	jmp NextLoop

SwitchOff	
	; Turn off second LED
	bis.b #080h,&P3OUT
	; Intentional fall-through...

NextLoop
	jmp LoopTop

Display_0
	mov #Display_1,R4

	mov.b #081h,&LCDMEM2	; S3:S2

	ret

Display_1
	mov #Display_2,R4
	
	mov.b #081h,&LCDMEM2	; S3:S2
	ret

Display_2
	mov #Display_3,R4

	mov.b #081h,&LCDMEM2	; S3:S2
	ret

Display_3
	mov #Display_4,R4

	mov.b #082h,&LCDMEM2	; S1:S0	
	ret

Display_4
	mov #Display_5,R4

	mov.b #082h,&LCDMEM2	; S1:S0	
	ret

Display_5
	mov #Display_6,R4

	mov.b #082h,&LCDMEM2	; S1:S0	
	ret

Display_6
	mov #Display_7,R4

	mov.b #084h,&LCDMEM2	; S3:S2
	ret

Display_7
	mov #Display_8,R4

	mov.b #084h,&LCDMEM2	; S3:S2
	ret

Display_8
	mov #Display_9,R4

	mov.b #084h,&LCDMEM2	; S3:S2
	ret

Display_9
	mov #Display_10,R4

	mov.b #088h,&LCDMEM2	; S3:S2
	ret

Display_10
	mov #Display_11,R4

	mov.b #088h,&LCDMEM2	; S3:S2
	ret

Display_11
	mov #Display_0,R4

	mov.b #088h,&LCDMEM2	; S3:S2
	ret

Init_UART
	; Assumes bis.b #003h,&P4SEL elsewhere
	; Setup Tx pin direction
	bis.b #BIT0,&P4DIR
	; Rx pin is input

	; Enable module
	bis.b #(UTXE1|URXE1),&ME2

	; Setup 2400 baud with ACLK
	mov.b #00Dh,&U1BR0
	mov.b #000h,&U1BR1
	mov.b #06Bh,&U1MCTL
	
	; Reset UART
	mov.b #(CHAR),&U1CTL
	mov.b #(TXEPT+UART_CLK_ACLK),&U1TCTL
	mov.b #(URXEIE),&U1RCTL

	; Set Tx Flag
	bis.b #(UTXIFG1),&IFG2
		
	; Enable receive interrupts
	bis.b #(URXIE1),&IE2

	ret

Int_BT
	; Wake processor
	bic #(CPUOFF+OSCOFF+SCG1+SCG0),0(SP)
	reti
	
Int_URX1
; Handle received byte
	mov.b &U1RXBUF,R15
	inc R15
	mov.b R15,&U1TXBUF

	reti	

; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; Interrupt vectors
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *

	.sect	  "Int_Vect",USER_END-31
	.word	  Int_BT			  ; Basic Timer
	.word	  RESET				  ; no source
	.word	  RESET				  ; no source
	.word	  Int_URX1			  ; UART1 Rx
	.word	  RESET				  ; no source
	.word	  RESET				  ; no source
	.word	  RESET				  ; no source
	.word	  RESET				  ; no source
	.word	  RESET				  ; no source
	.word	  RESET				  ; no source
	.word	  RESET				  ; no source
	.word	  RESET				  ; no source
	.word	  RESET				  ; no source
	.word	  RESET				  ; no source
	.word	  RESET				  ; NMI, Osc. fault
	.word	  RESET				  ; POR, ext. Reset, Watchdog
	.end